import {Component, ElementRef, HostListener, Injector, OnDestroy, ViewChild} from '@angular/core';
import {Subscription} from 'rxjs';
import {DataSourceService} from './services/data-source.service';
import {FeedbackComponent} from './feedback/feedback.component';
import {UserContextService} from './services/user-context.service';
import {RoleSelectorComponent} from './role-selector/role-selector.component';
import {MatIconRegistry} from '@angular/material/icon';
import {DomSanitizer} from '@angular/platform-browser';
import {DataCard, DataCardResponse} from 'facade-api';
import {DialogService} from './modals/dialog/dialog.service';
import {PatientContextService} from './services/patient-context.service';

export class ServiceLoader{
  public static injector: Injector;
}

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnDestroy {
  title = 'eGraviditet';
  dataCard: DataCard | undefined;
  allDataCards: DataCardResponse[] = [];
  hasPatientInContext = false;
  hasEpisodeOfCareInContext = false;
  private dataCardSubscription?: Subscription;

  @ViewChild('sidebar') sidebar?: ElementRef<HTMLElement>;

  constructor(
    public dataSourceService: DataSourceService,
    public userContextService: UserContextService,
    public dialogService: DialogService,
    private patientContextService: PatientContextService,
    private matIconRegistry: MatIconRegistry,
    private domSanitizer: DomSanitizer,
    private injector: Injector
  ) {
    // We need the injector in the httpInterceptor, but we cant inject it since it would cause a cyclic reference
    // - so we store it in a static here
    ServiceLoader.injector = injector;
    this.registerIcons();
    this.userContextService.userContext$.pipe(
    );
    this.hasPatientInContext = !!this.patientContextService.getCurrentCpr();
    patientContextService.cpr$.subscribe((cpr) => {
      this.hasPatientInContext = !!cpr;
      if (cpr) {
        this.dataSourceService.refresh();
      }
    });

    this.hasEpisodeOfCareInContext = !!this.patientContextService.getCurrentEpisodeOfCare();
    patientContextService.episodeOfCare$.subscribe((episodeOfCareId) => {
      this.hasEpisodeOfCareInContext = !!episodeOfCareId;
      this.dataCard = this.allDataCards.find(x => x.episodeOfCareIdentifier === episodeOfCareId);
    });

    this.dataCardSubscription = this.dataSourceService.dummyDataCards$.subscribe((cards: DataCardResponse[]) => {
      this.allDataCards = cards;
      this.dataCard = cards[0];
      switch (cards[0].documentStatus) {
        case 'ACTIVE':
          this.patientContextService.setEpisodeOfCare(cards[0].episodeOfCareIdentifier);
          break;
        case 'NEW':
        case 'CLOSED':
          // TODO: add handlers here
      }
    });
  }

  private registerIcons(): void {
    this.matIconRegistry.addSvgIcon('ic-edit',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_edit.svg'));
    this.matIconRegistry.addSvgIcon('ic-view',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_view-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-edit-highlight',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_edit_highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-trash',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_trash.svg'));
    this.matIconRegistry.addSvgIcon('ic-calendar',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_calendar.svg'));
    this.matIconRegistry.addSvgIcon('ic-more',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_more.svg'));
    this.matIconRegistry.addSvgIcon('ic-info',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_info.svg'));
    this.matIconRegistry.addSvgIcon('ic-add',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_add.svg'));
    this.matIconRegistry.addSvgIcon('ic-close',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_close-large.svg'));
    this.matIconRegistry.addSvgIcon('ic-care-plan',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_fagperson-mand.svg'));
    this.matIconRegistry.addSvgIcon('ic-overview',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_overblik.svg'));
    this.matIconRegistry.addSvgIcon('ic-add-colored',
      this.domSanitizer.bypassSecurityTrustResourceUrl('../assets/img/ic_add-colored.svg'));
  }

  openChangeRoleDialog(): void {
   this.dialogService.show(RoleSelectorComponent, {
      width: 'fit-content',
    });

  }

  @HostListener('window:scroll')
  public allowYScrollFrame(): void {
    const supportPageOffset = window.pageXOffset !== undefined;
    const isCSS1Compat = ((document.compatMode || '') === 'CSS1Compat');
    const x = supportPageOffset ? window.pageXOffset : isCSS1Compat ? document.documentElement.scrollLeft : document.body.scrollLeft;
    const y = supportPageOffset ? window.pageYOffset : isCSS1Compat ? document.documentElement.scrollTop : document.body.scrollTop;
    if (this.sidebar?.nativeElement) {
      this.sidebar.nativeElement.style.left = `${-x}px`;
      if (y <= 72) {
        this.sidebar.nativeElement.style.top = `${72 - y}px`;
      }
    }
  }

  ngOnDestroy(): void {
    this.dataCardSubscription?.unsubscribe();
  }

  openFeedbackDialog(): void {
    this.dialogService.show(FeedbackComponent);
  }

  public closePatient(): void {
    console.log('TODO: this should close the patient');
  }
}
